
/** A simple application to demonstrate the Event Delegation Model */
import java.awt.*;
import java.awt.event.*;

public class SimpleWindow extends Frame {

    Button quitButton;                   // The source
    QuitHandler quitHandler;             // The listener

    public SimpleWindow() {

        // Create a window
        super("SimpleWindow");

        // Create one button
        quitButton = new Button("Quit");

        // Set a layout manager, and add the button to the window.
        setLayout(new FlowLayout(FlowLayout.CENTER));
        add(quitButton);

        // Create and add the listener to the button
        quitHandler = new QuitHandler(this);                    // (1)
        quitButton.addActionListener(quitHandler);              // (2)

        // Pack the window and pop it up.
        pack();
        setVisible(true);
    }

    /** Create an instance of the application */
    public static void main(String args[]) { new SimpleWindow(); }

}

// Definition of the Listener
class QuitHandler implements ActionListener {                   // (3)

    private SimpleWindow application;    // The associated application

    public QuitHandler(SimpleWindow window) {
        application = window;
    }

    // Invoked when the user clicks the quit button.
    public void actionPerformed(ActionEvent evt) {              // (4)
        if (evt.getSource() == application.quitButton) {        // (5)
            System.out.println("Quitting the application.");
            application.dispose();                              // (6)
            System.exit(0);                                     // (7)
        }
    }
}
