/* SimpleWindowTwo: A simple setup for Event Delegation Model */
import java.awt.*;
import java.awt.event.*;

/** A simple application to demonstrate the Event Delegation Model */
public class SimpleWindowTwo extends Frame {

    Button quitButton;                   // The source
    QuitHandler quitHandler;             // The listener

    public SimpleWindowTwo() {

        // Create a window
        super("SimpleWindow");

        // Create one button
        quitButton = new Button("Quit");

        // Set a layout manager, and add the button to the window.
        setLayout(new FlowLayout(FlowLayout.CENTER));
        add(quitButton);

        // Create and add the listener to the button
        quitHandler = new QuitHandler(this);                    // (1)
        quitButton.addActionListener(quitHandler);              // (2)

        // Add the listener to the window
        addWindowListener(quitHandler);                         // (3)

        // Pack the window and pop it up.
        pack();
        setVisible(true);
    }

    /** Create an instance of the application */
    public static void main(String args[]) { new SimpleWindowTwo(); }

}

// Definition of the Listener
class QuitHandler extends WindowAdapter implements ActionListener { // (4)

    private SimpleWindowTwo application;    // The associated application

    public QuitHandler(SimpleWindowTwo window) {
        application = window;
    }

    // Terminate the application.
    private void terminate() {                                  // (5)
        System.out.println("Quitting the application.");
        application.dispose();
        System.exit(0);
    }

    // Invoked when the user clicks the quit button.
    public void actionPerformed(ActionEvent evt) {
        if (evt.getSource() == application.quitButton) {
            terminate();
        }
    }

    // Invoked when the user clicks the close-box
    public void windowClosing(WindowEvent evt) {                // (6)
        terminate();
    }
}
