/* SimpleWindowFour: A simple setup for Event Processing */
import java.awt.*;
import java.awt.event.*;

/** A simple application to demonstrate Event Processing */
public class SimpleWindowFour extends Frame {

    QuitButton quitButton;

    public SimpleWindowFour() {

        // Create a window
        super("SimpleWindowFour");

        // Create one button
        quitButton = new QuitButton("Quit", this);             // (1)

        // Add the button to the window.
        add(quitButton);

        // Enable the required events
        enableEvents(AWTEvent.KEY_EVENT_MASK |
                     AWTEvent.WINDOW_EVENT_MASK);              // (2)

        // Pack the window and pop it up.
        pack();
        setVisible(true);
    }

    // Invoked when the user clicks the close-box.
    public void processWindowEvent(WindowEvent evt) {          // (3)
        if (evt.getID() == WindowEvent.WINDOW_CLOSING)
            terminate();

        super.processWindowEvent(evt);                         // (4)
    }

    // Invoked when the user types 'q' or 'Q'.
    public void processKeyEvent(KeyEvent evt) {                // (5)
        if (evt.getID() == KeyEvent.KEY_TYPED &&
            (evt.getKeyChar() == 'q' || evt.getKeyChar() == 'Q'))
            terminate();

        super.processKeyEvent(evt);                            // (6)
    }

    public void terminate() {
        System.out.println("Quitting the application.");
        dispose();
        System.exit(0);
    }

    /** Create an instance of the application */
    public static void main(String args[]) {                   // (7)
        // Create application
        SimpleWindowFour window = new SimpleWindowFour();
    }
}

class QuitButton extends Button {

    private SimpleWindowFour application;

    public QuitButton(String name, SimpleWindowFour window) {  // (8)
        super(name);
        application = window;
        enableEvents(AWTEvent.ACTION_EVENT_MASK |
                     AWTEvent.KEY_EVENT_MASK);                 // (9)
    }

    // Invoked when the user clicks the quit button.
    public void processActionEvent(ActionEvent evt) {          // (10)
        if (evt.getSource() == this)
           application.terminate();

        super.processActionEvent(evt);                         // (11)
    }

    // Invoked when the user types 'q' or 'Q'.
    public void processKeyEvent(KeyEvent evt) {                // (12)
        if (evt.getID() == KeyEvent.KEY_TYPED &&
            (evt.getKeyChar() == 'q' || evt.getKeyChar() == 'Q'))
            application.terminate();

        super.processKeyEvent(evt);                            // (13)
    }
}
