/* SimpleWindowFive: A simple setup for Event Processing */
import java.awt.*;
import java.awt.event.*;

/** A simple application to demonstrate Event Processing */
public class SimpleWindowFive extends Frame {

    QuitButton quitButton;

    public SimpleWindowFive() {

        // Create a window
        super("SimpleWindowFive");

        // Create one button
        quitButton = new QuitButton("Quit", this);

        // Add the button to the window.
        add(quitButton);

        // Enable the required events
        enableEvents(AWTEvent.KEY_EVENT_MASK |
                     AWTEvent.WINDOW_EVENT_MASK);

        // Pack the window and pop it up.
        pack();
        setVisible(true);
    }

    // Event processing
    public void processEvent(AWTEvent evt) {                 // (1)
        // Invoked when the user clicks the close-box.
        if (evt.getID() == WindowEvent.WINDOW_CLOSING)
            terminate();

        // Invoked when the user types 'q' or 'Q'.
        if (evt.getID() == KeyEvent.KEY_TYPED &&
            (((KeyEvent) evt).getKeyChar() == 'q' ||
            ((KeyEvent) evt).getKeyChar() == 'Q'))
            terminate();

        super.processEvent(evt);
    }

    public void terminate() {
        System.out.println("Quitting the application.");
        dispose();
        System.exit(0);
    }

    /** Create an instance of the application */
    public static void main(String args[]) {

        SimpleWindowFive window = new SimpleWindowFive();
    }

}

class QuitButton extends Button {

    private SimpleWindowFive application;

    public QuitButton(String name, SimpleWindowFive window) {
        super(name);
        application = window;
        enableEvents(AWTEvent.ACTION_EVENT_MASK |
                     AWTEvent.KEY_EVENT_MASK);
    }

    // Event processing
    public void processEvent(AWTEvent evt) {                 // (2)
        // Invoked when the user clicks the quit button.
        if ((evt instanceof ActionEvent) &&
            ((ActionEvent) evt).getSource() == this)
            application.terminate();

        // Invoked when the user types 'q' or 'Q'.
        if (evt.getID() == KeyEvent.KEY_TYPED &&
            (((KeyEvent) evt).getKeyChar() == 'q' ||
            ((KeyEvent) evt).getKeyChar() == 'Q'))
             application.terminate();

        super.processEvent(evt);
    }
}
