
// Demo for usage of standard methods in an applet.
// Runs in its own thread, continuously drawing a Pac-man figure
// in two alternating colors.

import java.awt.*;
import java.awt.event.*;
import java.applet.*;

public class ThreadApplet extends Applet implements Runnable {

    private Thread appletThread = null;
    private boolean stopped = false;
    private int mouthGap = 0;

    public void init() {                                        // (1)
        // Set the foreground color
        setForeground(Color.green);

        // Alternate mouse clicks will stop or restart the painting.
        addMouseListener(new MouseAdapter() {                   // (2)
            public void mouseClicked(MouseEvent e) {
                toggleAnimation();
            }
        });
    }

    public void start() {                                       // (3)
        // Create and start a thread if none present.
        if (appletThread == null) {
            appletThread = new Thread(this);
            appletThread.start();
        }
    }

    public void stop() {                                        // (4)
        appletThread = null;
    }

    public void run() {                                         // (5)
        Thread thisThread = Thread.currentThread();
        while (thisThread == appletThread) {                    // (6)
            // Change foreground color while running.
            // Painting color will change accordingly.
            if(getForeground().equals(Color.green))
                setForeground(Color.blue);
            else
                setForeground(Color.green);

            // Update mouth gap
            mouthGap = (mouthGap+5) % 60;

            // Update the screen
            repaint();
            showStatus("Click to stop.");

            // Take a break between painting
            try {
                Thread.sleep(100);                              // (7)
                // If drawing is stopped, then wait.
                synchronized(this) {                            // (8)
                    while (stopped) {                           // (9)
                        showStatus("Click to restart.");
                        wait();                                 // (10)
                    }
                }
            } catch (InterruptedException e) {}
        }
    }

    public void paint(Graphics gfx) {                           // (11)
        // Draw a "Pac-man" in the current color
        int angle = Math.abs(mouthGap-30) + 5;  // Set mouth angle
        gfx.fillArc (50, 50, 50, 50, angle, 360 - angle*2);
    }

    private synchronized void toggleAnimation() {               // (12)
        // Check if the thread is present.
        if (appletThread != null && appletThread.isAlive()) {

            // Toggle between stopping and resuming the painting
            stopped = !stopped;                                 // (13)

            // If drawing should restart, then notify.
            if (!stopped) {
                notify();                                       // (14)
            }
        } else { // No thread. Create one and start it.            (15)
            stopped = false;
            start();
        }
    }
}
