import javax.swing.*;
import javax.swing.tree.DefaultMutableTreeNode;     // Used as nodes in JTree.
import java.util.*;                           // Map used while creating tree.

public class SolarSystem {
    public static void main(String args[]) {

        // Data to initialize the components:                             (1)
        final int NAME     = 0;    // Index constants for accessing
        final int ORBITING = 3;    // data in the following arrays.
        Object[] columnNames =
            {"Planet", "Orbit (km)",          "Diameter (km)",    "Orbiting"};
        Object[][] planetaryData = {
            {"Sun",     null,                 new Float(1390000), null   },
            {"Mercury", new Float(57910000),  new Float(4880),    "Sun"  },
            {"Venus",   new Float(108200000), new Float(12103.6), "Sun"  },
            {"Earth",   new Float(149600000), new Float(12756.3), "Sun"  },
            {"Moon",    new Float(384400),    new Float(3476),    "Earth"},
            {"Mars",    new Float(227940000), new Float(6794),    "Sun"  },
            {"Phobos",  new Float(9378),      new Float(22.2),    "Mars" },
            {"Deimos",  new Float(23459),     new Float(12.6),    "Mars" },
        };  // The last column indicates a parent-child relationship.

        // Build tree nodes based on data.                                (2)
        Map namesToNodes = new HashMap();  // mapping names to nodes.

        for (int i=0; i<planetaryData.length; i++) {    // For all planets...
            Object[] planetProfile = planetaryData[i];       // lookup data

            DefaultMutableTreeNode newNode =                 // make new node
                new DefaultMutableTreeNode(planetProfile[NAME]);

            namesToNodes.put(planetProfile[NAME], newNode);  // store node

            DefaultMutableTreeNode parent = (DefaultMutableTreeNode)
                namesToNodes.get(planetProfile[ORBITING]);   // find parent

            if (parent != null) parent.add(newNode);         // add to parent
        }

        // Find node for "Sun", which will be used as the root of the JTree
        DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode)
            namesToNodes.get("Sun");

        // Construct and display the GUI:                                 (3)
        JTree tree = new JTree(rootNode);
        JTable table = new JTable(planetaryData, columnNames);

        JScrollPane scrollableTree  = new JScrollPane(tree);
        JScrollPane scrollableTable = new JScrollPane(table);

        JSplitPane windowContents = new              // Split window contents 
            JSplitPane(JSplitPane.HORIZONTAL_SPLIT,  // horizontally,
                       scrollableTree,               // place tree left,
                       scrollableTable);             // place table right.

        JFrame mainWindow = new JFrame("Illustrating complex components");
        mainWindow.setContentPane(windowContents);

        mainWindow.pack();
        mainWindow.setVisible(true);
    }
}
