
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

public class SwingActions {
    public static void main(String args[]) {
        final JLabel amountLabel = new JLabel("Amount: 0");

        // Create a bounded range model                                 (1)
        final BoundedRangeModel amountModel =
            new DefaultBoundedRangeModel(0, 0, 0, 100) {
                public void setValue(int n) {
                    super.setValue(n);
                    amountLabel.setText("Amount: " + n);
                }
            };

        // Create actions that the user can activate                    (2)
        Action quitOperation = new AbstractAction("Quit") {
            public void actionPerformed(ActionEvent evt) { System.exit(0); }
        };
        Action fullOperation = new AbstractAction("Set full") {
            public void actionPerformed(ActionEvent evt) {
                amountModel.setValue(amountModel.getMaximum());
            }
        };
        Action emptyOperation = new AbstractAction("Set empty") {
            public void actionPerformed(ActionEvent evt) {
                amountModel.setValue(amountModel.getMinimum());
            }
        };

        // Create the toolbar and add the actions                       (3)
        JToolBar toolbar = new JToolBar();
        toolbar.add(quitOperation);
        toolbar.add(fullOperation);
        toolbar.add(emptyOperation);

        // Create the menu and add the actions                          (4)
        JMenu menu = new JMenu("File");
        menu.add(fullOperation);
        menu.add(emptyOperation);
        menu.addSeparator();
        menu.add(quitOperation);

        // Put the menu in the menu bar
        JMenuBar menubar = new JMenuBar();
        menubar.add(menu);

        // Share the model between a slider and a progress bar          (5)
        JSlider amountSlider = new JSlider(amountModel);
        JProgressBar progressBar = new JProgressBar(amountModel);

        // Pack components in a vertical box
        Box box = new Box(BoxLayout.Y_AXIS);
        box.add(amountSlider);
        box.add(progressBar);
        box.add(amountLabel);

        // Create the root component and compose its content pane
        JFrame window = new JFrame("Swing event handling");
        window.getContentPane().add(toolbar, BorderLayout.NORTH);
        window.getContentPane().add(box);
        window.getRootPane().setJMenuBar(menubar);

        window.addWindowListener(new WindowAdapter() {
            public void windowClosing(WindowEvent wEvt) {
                System.exit(0);
            }
        });
        window.pack();
        window.setVisible(true);
    }
}
