import java.util.*;

public class UniqueCharacterCounter {

    /**
     * A cache, mapping strings to count results. The count values are
     * stored as Integer objects within the map.
     */
    static Map globalCache = new HashMap();

    public static int countUniqueCharacters(String aString) {
        Object cachedResult = globalCache.get(aString);
        if (cachedResult != null)
            return ((Integer) cachedResult).intValue();

        // Result was not in the cache, calculate it.
        int length = aString.length();
        Set occured = new TreeSet();
        Set duplicates = new TreeSet();

        // Identify occurances and duplicates for each character in string:
        for (int i=0; i<length;i++) {
            Character character = new Character(aString.charAt(i));
            if (duplicates.contains(character)) continue;
            boolean newOccurance = occured.add(character);
            if (!newOccurance) duplicates.add(character);
        }

        // Remove duplicates from occurrence count to obtain result:
        occured.removeAll(duplicates);
        int result = occured.size();

        // Put result in cache before returning:
        globalCache.put(aString, new Integer(result));
        return result;
    }

    /**
     * A simple main method for the purpose of demonstrating the
     * effect of the <code>countUniqueCharacters()</code>
     * method. Prints the result of applying the operation on each
     * commandline argument.
     */
    public static void main(String args[]) {
        int nArgs = args.length;
        for (int i=0; i<nArgs; i++) {
            String argument = args[i];
            int result = countUniqueCharacters(argument);
            System.out.println(argument + ": " + result);
        }
    }
}
