// Filename: Pad.java
import java.awt.*;
import java.awt.event.*;

/** Implements a simple note pad. */
public class Pad {
    /** Simply create an instance of Pad */
    public static void main(String args[]) {
        new Pad();
    }

    String rememberedString;
    TextArea textEditArea;
    Label status;

    public Pad() {
        Window window = new Frame("Pad");

        textEditArea   = new TextArea();
        status         = new Label();

        window.add(textEditArea, BorderLayout.CENTER);
        window.add(status, BorderLayout.SOUTH);
        window.add(createButtons(), BorderLayout.NORTH);

        // Make the application exit when the window is closed by the user.
        window.addWindowListener(
            new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    System.exit(0);
                }
            }
        );

        window.pack();
        window.setVisible(true);
    }

    /** Create a row of command buttons. */
    public Component createButtons() {
        Panel buttonPanel = new Panel();

        Button clearButton    = new Button("Clear");
        Button rememberButton = new Button("Remember");
        Button recallButton   = new Button("Recall");

        buttonPanel.add(clearButton);
        buttonPanel.add(rememberButton);
        buttonPanel.add(recallButton);

        // Provide help for the command buttons.
        new ButtonHelp(clearButton,    "Clear text");
        new ButtonHelp(rememberButton, "Remember text");
        new ButtonHelp(recallButton,   "Recall text");

        clearButton.addActionListener(
            new ActionListener() {
                // Clear command implementation.
                public void actionPerformed(ActionEvent e) {
                    textEditArea.setText("");
                    setStatus("Text cleared");
                }
            }
        );

        rememberButton.addActionListener(
            new ActionListener() {
                // Remember command implementation.
                public void actionPerformed(ActionEvent e) {
                    rememberedString = textEditArea.getText();
                    setStatus("Text remembered");
                }
            }
        );

        recallButton.addActionListener(
            new ActionListener() {
                // Recall command implementation.
                public void actionPerformed(ActionEvent e) {
                    if (rememberedString == null) return;
                    textEditArea.setText(rememberedString);
                    setStatus("Text recalled");
                }
            }
        );

        return buttonPanel;
    }

    /** Set the message of the status line. */
    void setStatus(String msg) {
        status.setText(msg);
    }

    /** Clear any message on the status line. */
    void clearStatus() {
        setStatus("");
    }

    /**
     * A listener that will provide the user with help concerning the
     * command buttons.
     */
    class ButtonHelp extends MouseAdapter {
        ButtonHelp(Button button, String msg) {
            helpMsg = msg;
            button.addMouseListener(this);
        }

        String helpMsg;

        public void mouseEntered(MouseEvent e) {
            setStatus(helpMsg);
        }

        public void mouseExited(MouseEvent e) {
            clearStatus();
        }
    }
}
