// Filename: BorderPanel.java
import java.awt.*;

/**
 * A panel with a border to provide a visual clue to the grouping of the
 * components within it.
 */
public class BorderPanel extends Panel {
    static final int OUTER_PAD = 5;
    static final int INNER_PAD = 5;
    static final int PAD = OUTER_PAD + INNER_PAD;

    /**
     * Constructor corresponding to Panel()
     */
    public BorderPanel() {
        super();
    }

    /**
     * Constructor corresponding to Panel(LayoutManager)
     */
    public BorderPanel(LayoutManager layoutManager) {
        super(layoutManager);
    }

    public Insets getInsets() {
        /*
          Add additional padding to the inherited insets to
          accommodate for the space occupied by the border.
         */
        Insets insets = super.getInsets();
        return new Insets(insets.top+PAD, insets.left+PAD,
                          insets.bottom+PAD, insets.right+PAD);
    }

    public void paint(Graphics gfx) {
        /*
          The paint method of Panel usually does nothing, but call
          it anyway, just to be sure.
         */
        super.paint(gfx);

        /*
          Calculate the area where the border should be painted,
          based on padding constants and inherited insets.
         */
        Insets ins = super.getInsets();
        Dimension dim = getSize();

        int x, y, w, h;
        x = ins.left + OUTER_PAD;
        y = ins.top  + OUTER_PAD;

        w = dim.width  - ins.left - ins.right  - OUTER_PAD*2 - 1;
        h = dim.height - ins.top  - ins.bottom - OUTER_PAD*2 - 1;

        /*
          Paint an etched border that matches the color of the
          panel.
         */
        gfx.setColor(getBackground());
        gfx.draw3DRect(x-1, y-1, w+2, h+2, false);
        gfx.draw3DRect(x+1, y+1, w-2, h-2, true);
    }
}
