// Filename: SwingPad.java
import javax.swing.*;
import java.awt.*;
import java.awt.event.*;

/** Implements a simple note pad using Swing components. */
public class SwingPad {
    /** Simply creates an instance of SwingPad */
    public static void main(String args[]) {
        new SwingPad();
    }

    String rememberedString;
    JTextArea textEditArea;

    public SwingPad() {
        JFrame window = new JFrame("Swing pad");
        Container contentPane = window.getContentPane();

        /* Create a text area, wrap it in a scroll pane and stuff it
           in the content pane. */
        textEditArea = new JTextArea();
        JScrollPane editArea = new JScrollPane(textEditArea);
        contentPane.add(editArea, BorderLayout.CENTER);

        // Make the application exit when the window is closed by the user.
        window.addWindowListener(new WindowAdapter() {
                public void windowClosing(WindowEvent e) {
                    System.exit(0);
                }
        });

        // Create actions that the user can activate
        Action clearOperation = new AbstractAction("Clear") {
            public void actionPerformed(ActionEvent e) {
                textEditArea.setText("");
            }
        };

        Action rememberOperation = new AbstractAction("Remember") {
            public void actionPerformed(ActionEvent e) {
                rememberedString = textEditArea.getText();
            }
        };

        Action recallOperation = new AbstractAction("Recall") {
            public void actionPerformed(ActionEvent e) {
                if (rememberedString == null) return;
                textEditArea.setText(rememberedString);
            }
        };

        // Create the toolbar and add the actions
        JToolBar toolbar = new JToolBar();
        toolbar.add(clearOperation);
        toolbar.add(rememberOperation);
        toolbar.add(recallOperation);

        contentPane.add(toolbar, BorderLayout.NORTH);

        // Create the edit menu and add the actions
        JMenu editMenu = new JMenu("Edit");
        editMenu.add(clearOperation);
        editMenu.add(rememberOperation);
        editMenu.add(recallOperation);

        // Put the menu in the menu bar and attach it to menubar
        JMenuBar menubar = new JMenuBar();
        menubar.add(editMenu);
        window.getRootPane().setJMenuBar(menubar);

        window.pack();
        window.setVisible(true);
    }
}
