
// Filename: DrawRegion.java
import javax.swing.JPanel;
import java.awt.*;

/**
 * A drawing region that can store information about and paint a shape.
 */

class DrawRegion extends JPanel implements IGeometryConstants {   // (18)

    // Values needed for drawing the shape
    private int     shape;
    private int     xCoord;
    private int     yCoord;
    private boolean fillFlag;
    private int     width;

    /**
     * Creates a new DrawRegion instance.  The region is given a
     * default size and background color.
     */
    public DrawRegion() {
        setSize(300,300);
        setBackground(Color.white);
    }

    /**
     * Requests that the size of the drawing region remains constant.
     *
     * @return The current size of the component.
     */
    public Dimension getPreferredSize() {
        return getSize();
    }

    /**
     * Stores the values of a shape and schedules a repainting of the
     * drawing region.
     *
     * @param shape The type of shape as defined by the constants in
     *              {@link IGeometryConstants}.
     * @param xCoord The x coordinate of the shape.
     * @param yCoord The y coordinate of the shape.
     * @param fillFlag <code>true</code> if the shape should be filled.
     * @param width   The width/size of the shape.
     */
    public void doDraw(int shape,
                       int xCoord, int yCoord,
                       boolean fillFlag, int width) {

        this.shape    = shape;
        this.xCoord   = xCoord;
        this.yCoord   = yCoord;
        this.fillFlag = fillFlag;
        this.width    = width;

        repaint();                                                // (20)
    }

    /**
     * Paint the shape in the drawing region.
     *
     * @param g The graphics object that will be used to paint the shape.
     */
    public void paintComponent (Graphics g) {
        super.paintComponent(g);
        switch (shape) {
            case SQUARE:
                if (fillFlag) g.fillRect(xCoord, yCoord, width, width);
                else g.drawRect(xCoord, yCoord, width, width);
                break;
            case CIRCLE:
                if (fillFlag) g.fillOval(xCoord, yCoord, width, width);
                else g.drawOval(xCoord, yCoord, width, width);
                break;
            case ELLIPSE:
                if (fillFlag) g.fillOval(xCoord, yCoord, width, width/2);
                else g.drawOval(xCoord, yCoord, width, width/2);
                break;
        }
    }
}
